"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug_1 = __importDefault(require("debug"));
const createFlowDeclarationManager_1 = __importDefault(require("./createFlowDeclarationManager"));
const debug = debug_1.default('flowie:builder');
function buildPreparedFlowieManager(preparedFlowie, flowieContainer) {
    const [firstFlow, ...restOfFlow] = preparedFlowie.flows;
    const flowFunctionDetails = convertFlowElementToDeclarable(firstFlow, flowieContainer);
    const firstManager = createFlowDeclarationManager_1.default(flowFunctionDetails);
    const { preparedFlowieManager } = restOfFlow.reduce(parseFlows, {
        preparedFlowieManager: firstManager,
        flowieContainer
    });
    debug('Prepared flow built successfully!');
    return preparedFlowieManager;
}
exports.default = buildPreparedFlowieManager;
function parseFlows({ preparedFlowieManager, flowieContainer }, flowElement) {
    const flowFunctionDetailsList = convertFlowElementToDeclarable(flowElement, flowieContainer);
    if (flowElement.split) {
        return {
            flowieContainer,
            preparedFlowieManager: preparedFlowieManager.split(flowFunctionDetailsList)
        };
    }
    const [flowFunctionDetails] = flowFunctionDetailsList;
    return {
        flowieContainer,
        preparedFlowieManager: preparedFlowieManager.pipe(flowFunctionDetails)
    };
}
function convertFlowElementToDeclarable(flowElement, flowieContainer) {
    const pipeFlow = flowElement;
    if (pipeFlow.pipe) {
        if (typeof pipeFlow.pipe === 'string') {
            return [converToFlowFunctionDetails(pipeFlow.pipe, flowieContainer)];
        }
        return [
            buildPreparedFlowieManager({ flows: [pipeFlow.pipe], name: pipeFlow.name }, flowieContainer)
        ];
    }
    const splitFlow = flowElement;
    if (splitFlow.split) {
        const splitFlow = flowElement;
        return splitFlow.split.map(converSplitToFlowFunctionDetails, { flowieContainer });
    }
    return [buildPreparedFlowieManager(flowElement, flowieContainer)];
}
function converSplitToFlowFunctionDetails(flowieItemDeclaration) {
    if (typeof flowieItemDeclaration === 'string') {
        return converToFlowFunctionDetails(flowieItemDeclaration, this.flowieContainer);
    }
    return buildPreparedFlowieManager({ flows: [flowieItemDeclaration], name: flowieItemDeclaration.name }, this.flowieContainer);
}
function converToFlowFunctionDetails(functionName, flowieContainer) {
    const { isAsync, isGenerator } = flowieContainer.functionsContainer[functionName];
    return {
        name: functionName,
        isAsync,
        isGenerator
    };
}
//# sourceMappingURL=buildFlowDeclaration.js.map