"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function createFlowDeclarationManager(flowDeclarationOrFunctionList, preparedFlowieExecution) {
    const flowDeclaration = addNextItemToFlowDeclaration(preparedFlowieExecution, flowDeclarationOrFunctionList);
    return createFlowieDeclarationManagerRuntime(flowDeclaration);
}
exports.default = createFlowDeclarationManager;
function createFlowieDeclarationManagerRuntime(preparedFlowieExecution) {
    return Object.freeze({
        ...preparedFlowieExecution,
        pipe(flowFunctionDetailsOrflowieExecuteDeclaration) {
            const nextPreparedFlowieExecutionCandidate = flowFunctionDetailsOrflowieExecuteDeclaration;
            if (isFlowieExecutionDeclaration(nextPreparedFlowieExecutionCandidate)) {
                const nextPreparedFlowieExecution = mergePreparedFlowies(preparedFlowieExecution, nextPreparedFlowieExecutionCandidate);
                return createFlowieDeclarationManagerRuntime(nextPreparedFlowieExecution);
            }
            const flowFunctionDetails = flowFunctionDetailsOrflowieExecuteDeclaration;
            return createFlowDeclarationManager([flowFunctionDetails], preparedFlowieExecution);
        },
        split(declarationManagerOrFunctionDetailsList) {
            return createFlowDeclarationManager(declarationManagerOrFunctionDetailsList, preparedFlowieExecution);
        }
    });
}
function isFlowieExecutionDeclaration(preparedFlowieExecutionCandidate) {
    return typeof preparedFlowieExecutionCandidate.isAsync === 'boolean' &&
        Array.isArray(preparedFlowieExecutionCandidate.flows) &&
        preparedFlowieExecutionCandidate.allFunctionsNames instanceof Set;
}
exports.isFlowieExecutionDeclaration = isFlowieExecutionDeclaration;
function mergePreparedFlowies(preparedFlowieExecution, nextPreparedFlowieExecution) {
    return {
        isAsync: preparedFlowieExecution.isAsync || nextPreparedFlowieExecution.isAsync,
        allFunctionsNames: new Set([
            ...preparedFlowieExecution.allFunctionsNames,
            ...nextPreparedFlowieExecution.allFunctionsNames
        ]),
        flows: preparedFlowieExecution.flows.concat({ flows: nextPreparedFlowieExecution.flows })
    };
}
function addNextItemToFlowDeclaration(preparedFlowieExecution, flowDeclarationOrFunctionList) {
    const { isAsync, allFunctionsNames, flowElements } = flowDeclarationOrFunctionList.reduce(mergeFlowDeclarationAttributes, {
        isAsync: false,
        flowElements: [],
        allFunctionsNames: new Set()
    });
    if (!preparedFlowieExecution) {
        return {
            isAsync: isAsync,
            allFunctionsNames: allFunctionsNames,
            flows: [createFlowElementFromElements(flowElements)]
        };
    }
    return {
        isAsync: preparedFlowieExecution.isAsync || isAsync,
        allFunctionsNames: new Set([...preparedFlowieExecution.allFunctionsNames, ...allFunctionsNames]),
        flows: preparedFlowieExecution.flows.concat(createFlowElementFromElements(flowElements))
    };
}
function mergeFlowDeclarationAttributes(flowDeclarationAttributes, declarationManagerOrFunctionDetails) {
    return {
        isAsync: flowDeclarationAttributes.isAsync || declarationManagerOrFunctionDetails.isAsync,
        allFunctionsNames: new Set([
            ...flowDeclarationAttributes.allFunctionsNames,
            ...getFunctionNames(declarationManagerOrFunctionDetails)
        ]),
        flowElements: flowDeclarationAttributes.flowElements.concat(getFlowElement(declarationManagerOrFunctionDetails))
    };
}
function createFlowElementFromElements(flowElements) {
    if (flowElements.length === 1) {
        const [firstFlowElement] = flowElements;
        const preparedFlowie = firstFlowElement;
        if (preparedFlowie.flows) {
            return preparedFlowie;
        }
        return {
            pipe: firstFlowElement.name
        };
    }
    return {
        split: flowElements.map(getNameOrFlowieDeclaration)
    };
}
function getFunctionNames(declarationManagerOrFunctionDetails) {
    const flowDeclaration = declarationManagerOrFunctionDetails;
    if (isFlowieExecutionDeclaration(flowDeclaration)) {
        return Array.from(flowDeclaration.allFunctionsNames);
    }
    const flowFunctionDetails = declarationManagerOrFunctionDetails;
    return [flowFunctionDetails.name];
}
function getFlowElement(declarationManagerOrFunctionDetails) {
    const flowDeclaration = declarationManagerOrFunctionDetails;
    if (isFlowieExecutionDeclaration(flowDeclaration)) {
        return { flows: flowDeclaration.flows, name: flowDeclaration.name };
    }
    return declarationManagerOrFunctionDetails;
}
function getNameOrFlowieDeclaration(flowElement) {
    const preparedFlowie = flowElement;
    if (preparedFlowie.flows) {
        return preparedFlowie;
    }
    return flowElement.name;
}
//# sourceMappingURL=createFlowDeclarationManager.js.map