"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function reportFunctionCall(functionToCall, functionNameToReport, argument) {
    return reportFunction(functionToCall, functionNameToReport, argument);
}
exports.reportFunctionCall = reportFunctionCall;
async function reportAsyncFunctionCall(functionToCall, functionNameToReport, argument) {
    return reportAsyncFunction(functionToCall, functionNameToReport, argument);
}
exports.reportAsyncFunctionCall = reportAsyncFunctionCall;
function reportFunctionCallContext(functionToCall, functionNameToReport, argument, context) {
    return reportFunction(functionToCall, functionNameToReport, argument, context);
}
exports.reportFunctionCallContext = reportFunctionCallContext;
async function reportAsyncFunctionCallContext(functionToCall, functionNameToReport, argument, context) {
    return reportAsyncFunction(functionToCall, functionNameToReport, argument, context);
}
exports.reportAsyncFunctionCallContext = reportAsyncFunctionCallContext;
function startGeneratorReport(functionName) {
    return buildNextGeneratorReporter({
        functionName,
        hrTime: process.hrtime(),
        iterationTime: 0
    });
}
exports.startGeneratorReport = startGeneratorReport;
function calculateHRTimeDifference(previousHRTime) {
    // eslint-disable-next-line functional/prefer-readonly-type
    const nextHRTime = process.hrtime(previousHRTime);
    const differenceTime = (nextHRTime[0] / 1000) + (nextHRTime[1] / 1000000);
    return differenceTime;
}
exports.calculateHRTimeDifference = calculateHRTimeDifference;
function compactFunctionReport(functionsReportList) {
    const flowFunctionsResultList = Object.fromEntries(functionsReportList.reduce(collectReport, new Map()).entries());
    return flowFunctionsResultList;
}
exports.compactFunctionReport = compactFunctionReport;
function reportFunction(functionToCall, functionNameToReport, ...argumentAndContext) {
    const initialHRTime = process.hrtime();
    const result = functionToCall(...argumentAndContext);
    const executionTime = calculateHRTimeDifference(initialHRTime);
    return [{ functionName: functionNameToReport, executionTime }, result];
}
async function reportAsyncFunction(functionToCall, functionNameToReport, ...argumentAndContext) {
    const initialHRTime = process.hrtime();
    const result = await functionToCall(...argumentAndContext);
    const executionTime = calculateHRTimeDifference(initialHRTime);
    return [{ functionName: functionNameToReport, executionTime }, result];
}
function nextGeneratorReport(report) {
    const iterationTime = calculateHRTimeDifference(report.hrTime);
    return buildNextGeneratorReporter({
        functionName: report.functionName,
        hrTime: process.hrtime(),
        iterationTime
    });
}
function buildNextGeneratorReporter(report) {
    return { report, next: nextGeneratorReport.bind(null, report) };
}
function collectReport(
// perfomance issue
// eslint-disable-next-line functional/prefer-readonly-type
functionsReport, report) {
    if (!functionsReport.has(report.functionName)) {
        functionsReport.set(report.functionName, emptyFunctionCallReport);
    }
    const savedReport = functionsReport.get(report.functionName);
    const functionReport = report;
    if ('executionTime' in functionReport) {
        return functionsReport.set(report.functionName, collectFunctionCall(savedReport, functionReport));
    }
    const generatorReport = report;
    return functionsReport.set(report.functionName, collectIterationReport(savedReport, generatorReport));
}
function collectIterationReport(savedReport, generatorReport) {
    const savedIterations = savedReport.iterations || emptyIterationReport;
    const count = savedIterations.count + 1;
    const totalIterationTime = round(savedIterations.totalIterationTime + generatorReport.iterationTime, 6);
    const iterations = {
        count,
        slowestIterationTime: Math.max(savedIterations.slowestIterationTime, generatorReport.iterationTime),
        averageIterationTime: round(totalIterationTime / count, 6),
        fastestIterationTime: Math.min(savedIterations.fastestIterationTime, generatorReport.iterationTime),
        totalIterationTime
    };
    return Object.assign({}, savedReport, { iterations });
}
function collectFunctionCall(savedReport, functionReport) {
    const totalExecutionTime = round(savedReport.totalExecutionTime + functionReport.executionTime, 6);
    const calls = savedReport.calls + 1;
    return Object.assign({
        calls,
        slowestExecutionTime: Math.max(savedReport.slowestExecutionTime, functionReport.executionTime),
        averageExecutionTime: round(totalExecutionTime / calls, 6),
        fastestExecutionTime: Math.min(savedReport.fastestExecutionTime, functionReport.executionTime),
        totalExecutionTime
    }, savedReport.iterations ? { iterations: savedReport.iterations } : {});
}
function round(numberToRound, decimalPoints) {
    return parseFloat(numberToRound.toFixed(decimalPoints));
}
const emptyFunctionCallReport = {
    calls: 0,
    slowestExecutionTime: 0,
    averageExecutionTime: 0,
    fastestExecutionTime: Infinity,
    totalExecutionTime: 0
};
const emptyIterationReport = {
    count: 0,
    slowestIterationTime: 0,
    averageIterationTime: 0,
    fastestIterationTime: Infinity,
    totalIterationTime: 0
};
//# sourceMappingURL=reporter.js.map